import airo_models
import numpy as np
from pydrake.math import RigidTransform
from pydrake.multibody.tree import ModelInstanceIndex
from pydrake.planning import RobotDiagramBuilder


def add_floor(
    robot_diagram_builder: RobotDiagramBuilder,
    x_size: float = 2.0,
    y_size: float = 2.0,
    thickness: float = 0.2,
) -> ModelInstanceIndex:
    """Add a floor to the robot diagram builder. The floor is a box with the
    given dimensions and thickness. It is welded such that it's top surface is
    at z=0.

    Args:
        robot_diagram_builder: The robot diagram builder to which the floor will be added.
        x_size: The size of the floor in along the global x-axis.
        y_size: The size of the floor in along the global y-axis.
        thickness: The thickness of the floor (could be relevant for collision checking).

    Returns:
        The floor index.
    """
    plant = robot_diagram_builder.plant()
    parser = robot_diagram_builder.parser()
    parser.SetAutoRenaming(True)

    world_frame = plant.world_frame()

    floor_urdf_path = airo_models.box_urdf_path((x_size, y_size, thickness), "floor")
    floor_transform = RigidTransform(p=np.array([0, 0, -thickness / 2]))
    floor_index = parser.AddModels(floor_urdf_path)[0]
    floor_frame = plant.GetFrameByName("base_link", floor_index)

    plant.WeldFrames(world_frame, floor_frame, floor_transform)

    return floor_index



from pydrake.geometry import Meshcat, MeshcatVisualizer, MeshcatVisualizerParams, Role
from pydrake.planning import RobotDiagramBuilder


def add_meshcat(robot_diagram_builder: RobotDiagramBuilder) -> Meshcat:
    scene_graph = robot_diagram_builder.scene_graph()
    builder = robot_diagram_builder.builder()

    # Adding Meshcat must be done before finalizing
    meshcat = Meshcat()
    MeshcatVisualizer.AddToBuilder(builder, scene_graph, meshcat)

    # Add visualizer for proximity/collision geometry
    collision_params = MeshcatVisualizerParams(role=Role.kProximity, prefix="collision", visible_by_default=False)
    MeshcatVisualizer.AddToBuilder(builder, scene_graph.get_query_output_port(), meshcat, collision_params)

    return meshcat



# init files are still required in modern python!
# https://peps.python.org/pep-0420/ introduced implicit namespace packages
# but for building and many toolings, you still need to have __init__ files (at least in the root of the package).
# e.g. if you remove this init file and try to build with pip install .
# you won't be able to import the dummy module.
from airo_drake.building.finish import finish_build
from airo_drake.building.floor import add_floor
from airo_drake.building.manipulator import add_manipulator
from airo_drake.building.meshcat import add_meshcat
from airo_drake.scene import DualArmScene, SingleArmScene

__all__ = [
    "add_floor",
    "add_manipulator",
    "add_meshcat",
    "finish_build",
    "SingleArmScene",
    "DualArmScene",
]
